@@ -1,10 +1,12 @@
+//! Defines [`AppendError`], [`AppendArgs`] and related types.
+
 use std::ops::RangeInclusive;
 
 use num_traits::Bounded;
 use num_traits::Zero;
 use thiserror::Error;
 
-use crate::error::BoxError;
+use crate::error::ShutDownOr;
 use crate::invocation::AbstainOf;
 use crate::invocation::CommunicationErrorOf;
 use crate::invocation::Invocation;
@@ -13,28 +15,46 @@ use crate::invocation::RoundNumOf;
 use crate::retry::DoNotRetry;
 use crate::retry::RetryPolicy;
 
-pub struct AppendArgs<I: Invocation> {
+/// Options that determine how an append is performed.
+#[derive(Debug)]
+pub struct AppendArgs<I: Invocation, R = DoNotRetry<I>> {
+    /// The round(s) in which the entry should be appended. The append won't be
+    /// started until the round(s) can be settled and will fail if they already
+    /// are.
     pub round: RangeInclusive<RoundNumOf<I>>,
+    /// The append's [importance][Importance].
     pub importance: Importance,
-    pub retry_policy: Box<dyn RetryPolicy<Invocation = I> + Send>,
+    /// The [retry policy][crate::retry::RetryPolicy] with which to perform the
+    /// append.
+    pub retry_policy: R,
 }
 
-impl<I: Invocation> Default for AppendArgs<I> {
+impl<I: Invocation, R: RetryPolicy<Invocation = I> + Default> Default for AppendArgs<I, R> {
     fn default() -> Self {
-        DoNotRetry::new().into()
+        R::default().into()
     }
 }
 
-impl<I, P> From<P> for AppendArgs<I>
+impl<I: Invocation, R: Clone> Clone for AppendArgs<I, R> {
+    fn clone(&self) -> Self {
+        Self {
+            round: self.round.clone(),
+            importance: self.importance,
+            retry_policy: self.retry_policy.clone(),
+        }
+    }
+}
+
+impl<I, R> From<R> for AppendArgs<I, R>
 where
     I: Invocation,
-    P: RetryPolicy<Invocation = I> + Send + 'static,
+    R: RetryPolicy<Invocation = I>,
 {
-    fn from(retry_policy: P) -> Self {
+    fn from(retry_policy: R) -> Self {
         Self {
             round: Zero::zero()..=Bounded::max_value(),
             importance: Importance::GainLeadership,
-            retry_policy: Box::new(retry_policy),
+            retry_policy,
         }
     }
 }
@@ -63,6 +83,12 @@ pub enum Importance {
     MaintainLeadership(Peeryness),
 }
 
+impl Default for Importance {
+    fn default() -> Self {
+        Self::GainLeadership
+    }
+}
+
 /// Whether to inquire with other nodes about the round in question.
 ///
 /// If a node has to abandon an append due to [lack of status][Maintain] it may
@@ -84,13 +110,10 @@ pub enum Peeryness {
     Unpeery,
 }
 
+/// Reason an append failed.
 #[derive(Error)]
 #[non_exhaustive]
 pub enum AppendError<I: Invocation> {
-    /// Append was aborted.
-    #[error("append was aborted")]
-    Aborted(BoxError),
-
     /// The chosen round had already converged.
     #[error("round had already converged")]
     Converged {
@@ -114,14 +137,17 @@ pub enum AppendError<I: Invocation> {
     /// Failed in achieving a quorum.
     #[error("node could not achieve a quorum")]
     NoQuorum {
+        /// All abstentions received.
         abstentions: Vec<AbstainOf<I>>,
+        /// All encountered errors.
         communication_errors: Vec<CommunicationErrorOf<I>>,
+        /// All rejections received.
         rejections: Vec<NayOf<I>>,
     },
 
     /// Catch-all, this may be refined over time.
     #[error("a node decoration raised an error")]
-    Decoration(#[source] crate::error::BoxError),
+    Decoration(#[source] Box<dyn std::error::Error + Send + Sync + 'static>),
 
     /// Node is in passive mode.
     #[error("node is passive")]
@@ -136,10 +162,19 @@ pub enum AppendError<I: Invocation> {
     ShutDown,
 }
 
+// TODO any way to implement this genericly as `impl From<ShutDownOr<E>> for E`?
+impl<I: Invocation> From<ShutDownOr<AppendError<I>>> for AppendError<I> {
+    fn from(e: ShutDownOr<AppendError<I>>) -> Self {
+        match e {
+            ShutDownOr::Other(e) => e,
+            ShutDownOr::ShutDown => AppendError::ShutDown,
+        }
+    }
+}
+
 impl<I: Invocation> std::fmt::Debug for AppendError<I> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            AppendError::Aborted(err) => f.debug_tuple("AppendError::Aborted").field(err).finish(),
             AppendError::Converged { caught_up } => f
                 .debug_struct("AppendError::Converged")
                 .field("caught_up", caught_up)
