@@ -119,9 +119,10 @@ where
                     .is_ok()
             });
 
-            if any_node_added_and_removed {
-                panic!("Simultaneously adding and removing a node is undefined.");
-            }
+            assert!(
+                !any_node_added_and_removed,
+                "Simultaneously adding and removing a node is undefined."
+            );
         }
 
         let prev = self
@@ -273,11 +274,11 @@ mod tests {
         }
 
         fn added_nodes(&self) -> Vec<N> {
-            self.0.clone().unwrap_or_else(Vec::new)
+            self.0.clone().unwrap_or_default()
         }
 
         fn removed_nodes(&self) -> Vec<N> {
-            self.1.clone().unwrap_or_else(Vec::new)
+            self.1.clone().unwrap_or_default()
         }
     }
 
