@@ -171,7 +171,7 @@ where
         offset: NonZeroUsize,
         now: Instant,
     ) -> bool {
-        assert!(offset <= state.concurrency());
+        assert!(offset <= crate::state::concurrency_of(state));
 
         let cluster = state.cluster_at(offset);
 
@@ -284,10 +284,7 @@ where
         self.decorated.handle()
     }
 
-    fn prepare_snapshot(
-        &self,
-    ) -> LocalBoxFuture<'static, Result<SnapshotFor<Self>, crate::error::PrepareSnapshotError>>
-    {
+    fn prepare_snapshot(&self) -> LocalBoxFuture<'static, SnapshotFor<Self>> {
         self.decorated.prepare_snapshot()
     }
 
@@ -360,6 +357,13 @@ where
     {
         self.decorated.append_impl(applicable, args)
     }
+
+    fn await_commit_of(
+        &self,
+        log_entry_id: crate::node::LogEntryIdOf<Self>,
+    ) -> LocalBoxFuture<'static, Result<crate::node::CommitFor<Self>, crate::error::ShutDown>> {
+        self.decorated.await_commit_of(log_entry_id)
+    }
 }
 
 impl<D, I> MasterLeasingNode<(I,)> for D
@@ -410,7 +414,7 @@ where
             }
 
             let state = self.read_stale().await.ok()?;
-            let majority = (1..=state.concurrency().into())
+            let majority = (1..=crate::state::concurrency_of(&*state).into())
                 .all(|o| self.has_majority_at_offset(&*state, NonZeroUsize::new(o).unwrap(), now));
 
             majority.then(|| state)
