@@ -59,7 +59,7 @@ pub type ContextOf<N> = invocation::ContextOf<InvocationOf<N>>;
 /// Shorthand to extract invocation's `CoordNum` type out of `N`.
 pub type CoordNumOf<N> = invocation::CoordNumOf<InvocationOf<N>>;
 /// Shorthand to extract invocation's `Event` type out of `N`.
-pub type EventOf<N> = invocation::EffectOf<InvocationOf<N>>;
+pub type EffectOf<N> = invocation::EffectOf<InvocationOf<N>>;
 /// Shorthand to extract `Invocation` type out of `N`.
 pub type InvocationOf<N> = <N as Node>::Invocation;
 /// Shorthand to extract state's `LogEntry` type out of `N`.
@@ -163,9 +163,7 @@ pub trait Node: Sized {
     fn handle(&self) -> NodeHandle<Self::Invocation>;
 
     /// Requests that snapshot of the node's current state be taken.
-    fn prepare_snapshot(
-        &self,
-    ) -> LocalBoxFuture<'static, Result<SnapshotFor<Self>, crate::error::PrepareSnapshotError>>;
+    fn prepare_snapshot(&self) -> LocalBoxFuture<'static, SnapshotFor<Self>>;
 
     /// Affirms that the given snapshot was written to persistent storage.
     ///
@@ -175,7 +173,10 @@ pub trait Node: Sized {
         snapshot: SnapshotFor<Self>,
     ) -> LocalBoxFuture<'static, Result<(), crate::error::AffirmSnapshotError>>;
 
-    /// Requests that given snapshot be installed.
+    /// Requests that the given snapshot be installed.
+    ///
+    /// The node will retain its current set of promises and preserve its mode
+    /// of participation.
     fn install_snapshot(
         &self,
         snapshot: SnapshotFor<Self>,
@@ -226,6 +227,12 @@ pub trait NodeImpl: Node {
         A: ApplicableTo<StateOf<Self>> + 'static,
         P: Into<AppendArgs<Self::Invocation, R>>,
         R: RetryPolicy<Invocation = Self::Invocation>;
+
+    /// Wait for the log entry with the given ID to be committed.
+    fn await_commit_of(
+        &self,
+        log_entry_id: LogEntryIdOf<Self>,
+    ) -> LocalBoxFuture<'static, Result<CommitFor<Self>, ShutDown>>;
 }
 
 /// Future returned by [`Node::next_event`].
@@ -272,8 +279,7 @@ where
     pub context: invocation::ContextOf<I>,
     pub node_id: invocation::NodeIdOf<I>,
     pub voter: V,
-    pub snapshot: Option<invocation::SnapshotFor<I>>,
-    pub force_passive: bool,
+    pub snapshot: invocation::SnapshotFor<I>,
     pub buffer: B,
     #[cfg(feature = "tracer")]
     pub tracer: Option<Box<dyn Tracer<I>>>,
