@@ -31,7 +31,6 @@ use crate::error::AffirmSnapshotError;
 use crate::error::CommitError;
 use crate::error::InstallSnapshotError;
 use crate::error::PrepareError;
-use crate::error::PrepareSnapshotError;
 use crate::error::ReadStaleError;
 use crate::event::DirectiveKind;
 use crate::event::Event;
@@ -135,7 +134,7 @@ impl<S: State, R: RoundNum, C: CoordNum> From<DeconstructedSnapshot<S, R, C>> fo
     fn from(s: DeconstructedSnapshot<S, R, C>) -> Self {
         Self {
             state_round: s.round,
-            state: Some(s.state),
+            state: s.state,
             greatest_observed_round_num: s.greatest_observed_round_num,
             greatest_observed_coord_num: s.greatest_observed_coord_num,
             participation: s.participation,
@@ -312,14 +311,13 @@ where
             let node_id = spawn_args.node_id;
             let voter = spawn_args.voter;
             let snapshot = spawn_args.snapshot;
-            let force_passive = spawn_args.force_passive;
             let applied_entry_buffer = spawn_args.buffer;
             #[cfg(feature = "tracer")]
             let tracer = spawn_args.tracer;
 
             // assume we're lagging
             let initial_status = snapshot
-                .as_ref()
+                .state()
                 .map(|_| NodeStatus::Lagging)
                 .unwrap_or(NodeStatus::Disoriented);
 
@@ -331,15 +329,7 @@ where
                 participation,
                 promises,
                 accepted_entries,
-            } = snapshot
-                .map(|s| Init::from(s.deconstruct()))
-                .unwrap_or_default();
-
-            let participation = if force_passive {
-                Participation::passive()
-            } else {
-                participation
-            };
+            } = Init::from(snapshot.deconstruct());
 
             let _ = start_result_sender
                 .send((initial_status, super::Participation::from(&participation)));
@@ -479,7 +469,7 @@ where
                     }
 
                     // Limit the range to the concurrency window.
-                    let concurrency_bound = into_round_num(state.concurrency());
+                    let concurrency_bound = into_round_num(crate::state::concurrency_of(&**state));
                     let range =
                         *range.start()..=std::cmp::min(round + concurrency_bound, *range.end());
 
@@ -557,7 +547,7 @@ where
     }
 
     fn shut_down(mut self) {
-        let snapshot = self.prepare_snapshot().ok();
+        let snapshot = self.prepare_snapshot();
         let mut emitter = self.event_emitter;
 
         let _ = futures::executor::block_on(emitter.send(ShutdownEvent::Final { snapshot }));
@@ -565,7 +555,7 @@ where
 
     fn handle_request_msg(&mut self, req: Request<I>, resp_sender: oneshot::Sender<Response<I>>) {
         let resp = match req {
-            Request::PrepareSnapshot => Response::PrepareSnapshot(self.prepare_snapshot()),
+            Request::PrepareSnapshot => Response::PrepareSnapshot(Ok(self.prepare_snapshot())),
 
             Request::AffirmSnapshot { snapshot } => {
                 Response::AffirmSnapshot(self.affirm_snapshot(snapshot))
@@ -609,7 +599,7 @@ where
                             let offset = std::num::NonZeroUsize::new(offset)
                                 .expect("Zero was ruled out via equality check");
 
-                            let bound = state.concurrency();
+                            let bound = crate::state::concurrency_of(&**state);
 
                             // We don't give out reservations past the concurrency bound.
                             assert!(offset <= bound);
@@ -761,10 +751,6 @@ where
         let _ = resp_sender.send(resp);
     }
 
-    fn become_passive(&mut self) {
-        self.participation = Participation::passive();
-    }
-
     fn passive_for(&self, round_num: RoundNumOf<I>) -> bool {
         match self.participation {
             Participation::Active => false,
@@ -779,7 +765,7 @@ where
                 if self
                     .greatest_observed_round_num
                     .unwrap_or_else(Bounded::max_value)
-                    > self.state_round + into_round_num(state.concurrency())
+                    > self.state_round + into_round_num(crate::state::concurrency_of(&**state))
                 {
                     NodeStatus::Lagging
                 } else if self.greatest_observed_coord_num == self.leadership.1 {
@@ -856,7 +842,8 @@ where
     fn deduce_node(&self, round_num: RoundNumOf<I>, coord_num: CoordNumOf<I>) -> Option<NodeOf<I>> {
         self.state.as_ref().and_then(|state| {
             if round_num > self.state_round
-                && round_num <= self.state_round + into_round_num(state.concurrency())
+                && round_num
+                    <= self.state_round + into_round_num(crate::state::concurrency_of(&**state))
             {
                 // TODO lots of duplication with NodeInner::determine_coord_num
                 let round_offset = crate::util::usize_delta(round_num, self.state_round);
@@ -896,21 +883,16 @@ where
             std::cmp::max(self.greatest_observed_coord_num, coord_num);
     }
 
-    fn prepare_snapshot(&mut self) -> Result<SnapshotFor<I>, PrepareSnapshotError> {
-        let state = self
-            .state
-            .as_ref()
-            .ok_or(PrepareSnapshotError::Disoriented)?;
-
-        Ok(Snapshot::new(
+    fn prepare_snapshot(&mut self) -> SnapshotFor<I> {
+        Snapshot::new(
             self.state_round,
-            Arc::clone(state),
+            self.state.as_ref().map(Arc::clone),
             self.greatest_observed_round_num,
             self.greatest_observed_coord_num,
             self.participation.clone(),
             self.promises.clone(),
             self.accepted_entries.clone(),
-        ))
+        )
     }
 
     // TODO this doesn't do anything and probably never will, consider removal
@@ -928,18 +910,14 @@ where
             state,
             greatest_observed_round_num,
             greatest_observed_coord_num,
-            promises,
             accepted_entries,
             ..
         } = snapshot.deconstruct();
 
-        self.become_passive();
-
         self.state_round = state_round;
-        self.state = Some(Arc::clone(&state));
+        self.state = state.as_ref().map(Arc::clone);
         self.greatest_observed_round_num = greatest_observed_round_num;
         self.greatest_observed_coord_num = greatest_observed_coord_num;
-        self.promises = promises;
         self.accepted_entries = accepted_entries;
 
         self.emit(Event::Install {
@@ -1268,6 +1246,35 @@ where
         self.pending_commits
             .insert(round_num, (gap_time, coord_num, entry));
 
+        if let Participation::Passive {
+            observed_proposals, ..
+        } = &mut self.participation
+        {
+            // To get out of passivity, we must fully observe `c` rounds, where `c` is the
+            // level of concurrency that was applicable to the first fully observed round.
+            //
+            // (Applicable to a round means right before an entry is applied. If, for
+            // instance, we fully observe round `r` with an applicable concurrency of `1`,
+            // then we may immediately come out of passivity. Even if the entry applied in
+            // `r` increases concurrency.)
+            //
+            // It is important to remember that our criteria for a "fully observed round"
+            // are that the same round and coordination number are observed in an accept and
+            // a commit. This allows us to deduce that the first fully observed round `r`
+            // had not been committed before. This in turn allows us to become active in
+            // round `r + c`, regardless of any potential concurrency increases between now
+            // (`r`) and then (`r + c`) because no node could have seen them before (`r` was
+            // not settled).
+            if observed_proposals.contains(&(round_num, coord_num)) {
+                if let Some(concurrency) = StateOf::<I>::concurrency(self.state.as_deref()) {
+                    let first_active_round = round_num + into_round_num(concurrency);
+                    self.participation = Participation::PartiallyActive(first_active_round);
+
+                    self.emit(Event::Activate(first_active_round));
+                }
+            }
+        }
+
         // Are we missing commits between this one and the last one applied?
         if round_num > self.state_round + One::one() {
             self.emit_gaps(self.state_round);
@@ -1302,33 +1309,6 @@ where
         while let Some((_, coord_num, entry)) = self.pending_commits.remove(&(round + One::one())) {
             round = round + One::one();
 
-            if let Participation::Passive {
-                observed_proposals, ..
-            } = &mut self.participation
-            {
-                // To get out of passivity, we must fully observe `c` rounds, where `c` is the
-                // level of concurrency that was applicable to the first fully observed round.
-                //
-                // (Applicable to a round means right before an entry is applied. If, for
-                // instance, we fully observe round `r` with an applicable concurrency of `1`,
-                // then we may immediately come out of passivity. Even if the entry applied in
-                // `r` increases concurrency.)
-                //
-                // It is important to remember that our criteria for a "fully observed round"
-                // are that the same round and coordination number are observed in an accept and
-                // a commit. This allows us to deduce that the first fully observed round `r`
-                // had not been committed before. This in turn allows us to become active in
-                // round `r + c`, regardless of any potential concurrency increases between now
-                // (`r`) and then (`r + c`) because no node could have seen them before (`r` was
-                // not settled).
-                if observed_proposals.contains(&(round, coord_num)) {
-                    let first_active_round = round + into_round_num(state.concurrency());
-                    self.participation = Participation::PartiallyActive(first_active_round);
-
-                    self.emit(Event::Activate(first_active_round));
-                }
-            }
-
             debug!("Applying entry {:?} at round {}.", entry, round);
 
             let awaiters = self.awaiters.remove(&entry.id()).unwrap_or_default();
